package cz.muni.fi.pb162.test3;

/**
 * Tato trida reprezentuje nejaky kus pradla. Kazde pradlo ma nejaky popis, 
 * stupen zaspineni a hmotnost.
 *
 * @author Petr Adamek
 */
public class Laundry {
    
    private double dirtyFactor;
    private double weight;
    private String description;
    
    /**
     * Vytvori novou instanci reprezentujici kus pradla s danym popisem a 
     * danou hmotnosti.
     *
     * @param weight hmotnost pradla
     * @param description popis pradla
     */
    public Laundry(double weight, String description) {
        if (weight <= 0) {
            throw new IllegalArgumentException("weight must be greater than 0");
        }
        if (description == null) {
            throw new NullPointerException("description");
        }
        this.weight = weight;
        this.description = description;
    }

    /**
     * Vrati stupen zaspineni. Zcela ciste pradlo ma stupen zaspineni 0, zcela spinave 
     * pradlo ma stupen zaspineni 1.
     *
     * @return stupen zaspineni
     */
    public double getDirtyFactor() {
        return dirtyFactor;
    }

    /**
     * Nastavi stupen zaspineni pradla na danou hodnotu.
     *
     * @throws IllegalArgumentException pokud je dana hodnota mimo povoleny rozsah.
     * @param dirtyFactor novy stupen zaspineni
     */
    public void setDirtyFactor(double dirtyFactor) {
        if (dirtyFactor > 1 || dirtyFactor < 0) {
            throw new IllegalArgumentException("dirtyFactor must be greater " +
                    "than or equals to 0 and less than or equals to 1");
        }
        this.dirtyFactor = dirtyFactor;
    }

    /**
     * Vrati hmotnost tohoto kusu pradla.
     *
     * @return hmotnost kusu pradla
     */
    public double getWeight() {
        return weight;
    }

    /**
     * Vrati popis tohoto kusu pradla.
     *
     * @return popis pradla
     */
    public String getDescription() {
        return description;
    }

    public String toString() {
        return  description + " (" + weight + " kg)";
    }
    
}
