package cz.muni.fi.pb162.test3;

import java.util.Collection;

/**
 * Toto rozhrani reprezentuje moderni pracku. Pracka je vybavena RFID cteckou 
 * a pokud je do ni vlozeno pradlo s RFID cipem, umi zkontrolovat, jestli se 
 * pouziva spravny program a jestli se nekombinuje bile pradlo s pradlem 
 * barevnym. Zaroven umi dane pradlo zvazit a tomu prizpusobit davku praciho 
 * prasku, ktera se spotrebuje.
 *
 * Trida implementujici toto rozhrani bude mit konstruktor s jednim parametrem 
 * typu double, ktery bude obsahovat maximalni mnozstvi pradla (v kg), ktere
 * je mozne do pracky vlozit (bez ohledu na zvoleny program). Tento parametr
 * musi byt kladny.
 *
 * @author Petr Adamek
 */
public interface WashingMachine {
    
    /**
     * Prida do zasobniku pracky dane mnozstvi praciho prasku. Pro zjednoduseni 
     * predpokladejte, ze ma zasobnik neomezenou velikost.
     *
     * @param amount mnozstvi praciho prasku v kilogramech
     * @throws IllegalArgumentException pokud je amount zaporne cislo nebo nula
     */
    void addWashingPowder(double amount); 
         
    /**
     * Vrati mnozstvi praciho prasku v zasobniku.
     *
     * @return mnozstvi praciho prasku v zasobniku (v kg)
     */
    double getWashingPowderAmount();

    /**
     * Prida dany kus pradla do pracky.
     * 
     * @param laundry kus pradla, ktery se ma pridat do pracky.
     * @throws cz.muni.fi.pb162.test3.TooMuchLaundryException pokud by pridanim 
     * tohoto kusu pradla byla prekrocena maximalni povolena hmotnost pradla 
     * v pracce.
     * @throws IllegalArgumentException pokud jiz dany kus pradla v pracce je
     */
    void addLaundry(Laundry laundry) throws TooMuchLaundryException;

    /**
     * Vrati kolekci obsahujuci vsechno pradlo v pracce. Metoda by nemela 
     * porusovat princip zapouzdreni.
     *
     * @return kolekci obsahujuci vsechno pradlo v pracce
     */
    Collection<Laundry> getLaundry();

    /**
     * Nastavi dany praci program.
     *
     * @param program pozadovany program
     */
    void setProgram(WashingProgram program);
            
    /**
     * <p>Spusti praci proces. Nejdrive se zkontroluje, zdali celkova hmotnost 
     * pradla neprekracuje maximalni mnozstvi pradla, ktere je mozne nastavenym 
     * programem vyprat. Pak se zkontroluji vsechny kusy pradla, ktere jsou 
     * vybaveny RFID cipem, jestli je mozne je vyprat pri pozadovane teplote a
     * jestli se nekombinuje bile a barevne pradlo. Pak se ze zasobniku odebere
     * pozadovane mnozstvi praciho prasku (to je dane zvolenym programem) a 
     * jednotlive kusy pradla se vyperou.</p>
     * 
     * <p>Vlastni prani probiha tak, ze se mira zaspineni snizi v zavislosti na 
     * ucinnosti praciho programu. Pokud je ucinnost 100 %, mira zaspineni 
     * klesne na nulu, pokud je ucinnost 0 %, mira zaspineni se nezmeni. Nova 
     * mira zaspineni se tak spocita podle nasledujiciho vzorce:</p>
     * 
     * <p>novaMiraZaspineni = staraMiraZaspineni * (1 - ucinnost)</p>
     * 
     * <p>Pozor! Mnozstvi praciho prasku se udava v kg prasku na jeden kilogram 
     * pradla. Stupen zaspineni i ucinnost programu je dana jako cislo v 
     * intervalu 0 az 1, coz reprezentuje 0 az 100 %. Metoda by mela selhat atomicky, 
     * tj. pokud dojde k chybe, prani se vubec nezahaji.</p>
     *
     * @throws cz.muni.fi.pb162.test3.TooMuchLaundryException pokud je v pracce 
     * vetsi mnozstvi pradla, nez jake dany program povoluje
     * @throws cz.muni.fi.pb162.test3.IncompatibleLaundryException pokud se 
     * kombinuje bile a barevne pradlo (to lze samozrejme zkontrolovat pouze u 
     * pradla s RFID cipem)
     * @throws cz.muni.fi.pb162.test3.WrongProgramException pokud je v pracce 
     * nejake pradlo, ktere neni mozne prat pri teplote vybrane zvolenym pracim 
     * programem
     * @throws cz.muni.fi.pb162.test3.WashingException pokud neni v zasobniku 
     * dostatek praciho prasku
     * @throws java.lang.IllegalStateException pokud jeste nebyl vybran program
     */
    void wash() throws TooMuchLaundryException, IncompatibleLaundryException, 
            WrongProgramException, WashingException;
    
}
