package cz.muni.fi.pb162.test3;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import junit.framework.TestCase;

public class WashingMachineTest extends TestCase {
    
    public WashingMachineTest(String testName) {
        super(testName);
    }
        
    public void testAddWashingPowder() {
        WashingMachine washer = new WashingMachineImpl(5);
        assertEquals("pracka na zacatku obsahuje nejaky praci prasek",
                0.0,washer.getWashingPowderAmount());
        
        washer.addWashingPowder(1);
        assertEquals("ted by melo byt v pracce 1 kg prasku... ",
                1.0,washer.getWashingPowderAmount());

        washer.addWashingPowder(2);
        assertEquals("ted by melo byt v pracce 3 kg prasku... ",
                3.0,washer.getWashingPowderAmount());
        
        try {
            washer.addWashingPowder(0);
            fail("Je mozne do pracky pridat nulove mnozstvi praciho prasku");
        } catch(IllegalArgumentException ex) {
            // This is ok
        }
        
        assertEquals(3.0,washer.getWashingPowderAmount());
        
        try {
            washer.addWashingPowder(-0.5);
            fail("Je mozne do pracky pridat zaporne mnozstvi praciho prasku");
        } catch(IllegalArgumentException ex) {
            // This is ok
        }
        
        assertEquals("metoda addWashingPowder neselhava atomicky",
                3.0,washer.getWashingPowderAmount());
        
    }
    
    private <T> void assertCollectionsEquals(String msg,Collection<T> actual, T ... expected) {
        Set expectedSet = new HashSet<T>(Arrays.asList(expected));
        Set actualSet = new HashSet<T>(actual);
        assertEquals(msg,expectedSet,actualSet);
    }
    
    public void testAddLaundry() throws Exception {

        WashingMachine washer = new WashingMachineImpl(1);
        Laundry jeans = new Laundry(0.4,"rifle");
        Laundry betterJeans = new LaundryWithRFID(0.4,"lepsi rifle",LaundryWithRFID.Color.COLORED,80);
        Laundry sweatshirt = new Laundry(0.2,"mikina");
        washer.addLaundry(jeans);
        washer.addLaundry(betterJeans);
        washer.addLaundry(sweatshirt);
        
        assertCollectionsEquals("obsah pracky neni takovy, jaky by mel byt",
                washer.getLaundry(),jeans,betterJeans,sweatshirt);
        
        try {
            washer.addLaundry(new Laundry(0.02,"kapesnik"));
            fail("ocekava se TooMuchLaundryException");
        } catch(TooMuchLaundryException ex) {
            // All is ok
        }
        
        assertCollectionsEquals("obsah pracky neni takovy, jaky by mel byt; " +
                "metoda addLaundry zrejme neselhava atomicky",
                washer.getLaundry(),jeans,betterJeans,sweatshirt);

        washer = new WashingMachineImpl(1);
        try {
            washer.addLaundry(new Laundry(1.1,"Zimni bunda"));
            fail("ocekava se TooMuchLaundryException");
        } catch(TooMuchLaundryException ex) {
            // All is ok
        }

        assertCollectionsEquals("obsah pracky neni takovy, jaky by mel byt; " +
                "metoda addLaundry zrejme neselhava atomicky",
                washer.getLaundry());

        try {
            washer.addLaundry(jeans);
        } catch(TooMuchLaundryException ex) {
            fail("Metoda addLaundry neselhava atomicky");
        }

        assertCollectionsEquals("obsah pracky neni takovy, jaky by mel byt; " +
                "metoda addLaundry zrejme neselhava atomicky",
                washer.getLaundry(),jeans);
        
        try {
            washer.addLaundry(jeans);
            fail("do pracky je mozne jeden kus pradla dat dvakrat");
        } catch(IllegalArgumentException ex) {
            // All is ok
        }
        
    }

    public void testNewWashingMachineImpl() {
        WashingMachine washer = new WashingMachineImpl(5);
        assertEquals("pracka na zacatku obsahuje nejaky praci prasek",
                0.0,washer.getWashingPowderAmount());
        assertTrue("pracka na zacatku neni prazdna",washer.getLaundry().isEmpty());
        try {
            new WashingMachineImpl(0);
            fail("Je mozne vytvorit pracku s nulovou kapacitou pradla");
        } catch(IllegalArgumentException ex) {
            // This is ok
        }
        
        try {
            new WashingMachineImpl(-1);
            fail("Je mozne vytvorit pracku se zapornou kapacitou pradla");
        } catch(IllegalArgumentException ex) {
            // This is ok
        }
    }
    
    public void testGetLaundry() {
        WashingMachine washer = new WashingMachineImpl(5);
        Collection<Laundry> result = washer.getLaundry();
        assertTrue("pracka na zacatku neni prazdna",result.isEmpty());
        try {
            result.add(new Laundry(0.4,"rifle"));
            // result is not readonly collection, so we have to check if 
            // its change doesn't affect washer content
            assertTrue("metoda getLaundry umoznuje porusit princip zapouzdreni",
                    washer.getLaundry().isEmpty());
        } catch (UnsupportedOperationException ex) {
            // all is ok, result is readonly collection
        }
    }

    public void testWash() throws Exception {

        WashingMachine washer = new WashingMachineImpl(5);
        
        Laundry jeans = new Laundry(0.4,"rifle");
        Laundry betterJeans = new LaundryWithRFID(0.4,"lepsi rifle",LaundryWithRFID.Color.COLORED,80);
        Laundry sweatshirt = new Laundry(0.1,"lehka mikina");
        Laundry sweater = new LaundryWithRFID(0.4,"svetr",LaundryWithRFID.Color.COLORED,40);
        Laundry tShirt = new LaundryWithRFID(0.4,"tricko",LaundryWithRFID.Color.WHITE,60);

        jeans.setDirtyFactor(1);
        betterJeans.setDirtyFactor(0.5);
        sweatshirt.setDirtyFactor(0);
        
        washer.addLaundry(jeans);
        washer.addLaundry(betterJeans);
        washer.addLaundry(sweatshirt);
 
        try {
            washer.wash();
            fail("Lze spustit prani, aniz by byl vybran nejaky program; " +
                    "ocekava se IllegalStateException");
        } catch(IllegalStateException ex) {
            // All is ok
        }
 
        washer.setProgram(new WashingProgram(0.9,60,0.1,0.8));
        washer.addWashingPowder(0.08);

        try {
            washer.wash();
            fail("Lze spustit prani, aniz by byl v pracce dostatek prasku; " +
                    "ocekava se WashingException");
        } catch(WashingException ex) {
            // All is ok
        }

        assertEquals("metoda wash() neselhava atomicky",
                1.0,jeans.getDirtyFactor());
        assertEquals("metoda wash() neselhava atomicky",
                0.5,betterJeans.getDirtyFactor());
        assertEquals("metoda wash() neselhava atomicky",
                0.0,sweatshirt.getDirtyFactor());

        washer.addWashingPowder(0.92);

        washer.wash();
        
        assertEquals("rifle nemaji po vyprani ocekavany faktor zaspineni",
                0.2,jeans.getDirtyFactor(),0.00001);
        assertEquals("lepsi rifle nemaji po vyprani ocekavany faktor zaspineni",
                0.1,betterJeans.getDirtyFactor(),0.00001);
        assertEquals("mikina nema po vyprani ocekavany faktor zaspineni",
                0.0,sweatshirt.getDirtyFactor(),0.00001);
        
        assertEquals("Nebylo spotrebovano ocekavane mnozstvi prasku (0.09 kg) a v " +
                "pracce ho tudiz zbylo jine nez ocekavane mnozstvi",
                0.91,washer.getWashingPowderAmount());
        
    }

    public void testWashExceptions() throws Exception {

        
        Laundry jeans = new Laundry(0.4,"rifle");
        Laundry betterJeans = new LaundryWithRFID(0.4,"lepsi rifle",LaundryWithRFID.Color.COLORED,80);
        Laundry sweater = new LaundryWithRFID(0.4,"svetr",LaundryWithRFID.Color.COLORED,40);
        Laundry tShirt = new LaundryWithRFID(0.2,"tricko",LaundryWithRFID.Color.WHITE,60);

        jeans.setDirtyFactor(1);
        betterJeans.setDirtyFactor(0.5);
        sweater.setDirtyFactor(0);
        
        WashingMachine washer = new WashingMachineImpl(5);
        washer.addLaundry(jeans);
        washer.addLaundry(betterJeans);
        washer.setProgram(new WashingProgram(0.5,60,0.1,0.8));
        washer.addWashingPowder(1);

        try {
            washer.wash();
            fail("Lze spustit prani, ackoliv je v pracce vice pradla, nez kolik " +
                    "dany program podporuje; ocekava se TooMuchLaundryException");
        } catch(TooMuchLaundryException ex) {
            // All is ok
        }

        washer = new WashingMachineImpl(5);
        washer.addLaundry(jeans);
        washer.addLaundry(betterJeans);
        washer.addLaundry(sweater);
        washer.setProgram(new WashingProgram(2,60,0.1,0.8));
        washer.addWashingPowder(1);

        try {
            washer.wash();
            fail("Lze spustit prani, ackoliv je v pracce pradlo s RFID cipem, " +
                    "hlasici, ze se nesmi prat pri pozadovane teplote; ocekava " +
                    "se WrongProgramException");
        } catch(WrongProgramException ex) {
            // All is ok
        }


        washer = new WashingMachineImpl(5);
        washer.addLaundry(jeans);
        washer.addLaundry(betterJeans);
        washer.addLaundry(tShirt);
        washer.setProgram(new WashingProgram(2,40,0.1,0.8));
        washer.addWashingPowder(1);

        try {
            washer.wash();
            fail("Lze spustit prani, ackoliv je v pracce zkombinovano barevne a " +
                    "bile pradlo s RFID cipem; ocekava se IncompatibleLaundryException");
        } catch(IncompatibleLaundryException ex) {
            // All is ok
        }


    }

}
